import React, { useState } from "react";
import { useLocation, useNavigate } from "react-router-dom";
import "./SearchResults.css"; // Ensure to import the CSS file

const SearchResults = () => {
  const location = useLocation();
  const navigate = useNavigate();
  const initialUsers = location.state?.users || []; // Get users from state
  const [users, setUsers] = useState(initialUsers); // State to manage users
  const [editingUser, setEditingUser] = useState(null); // State to manage the user being edited
  const [formData, setFormData] = useState({}); // State to hold form data for the user

  const handleBackClick = () => {
    navigate(-1); // Navigate back to the previous page
  };

  const handleDeleteUser = (userId) => {
    const confirmDelete = window.confirm("Are you sure you want to delete this user?");
    if (confirmDelete) {
      // Filter out the user to be deleted from the state
      setUsers((prevUsers) => prevUsers.filter(user => user.id !== userId));
      console.log(`User with ID ${userId} deleted`);
      // You may also want to call your API to delete the user from the backend
    } else {
      console.log("User deletion canceled");
    }
  };

  const handleUpdateUser = (userId) => {
    const userToUpdate = users.find(user => user.id === userId);
    setEditingUser(userToUpdate);
    setFormData({ ...userToUpdate }); // Populate the form data with the user's current information
  };

  const handleInputChange = (e) => {
    const { name, value } = e.target;
    setFormData((prevData) => ({ ...prevData, [name]: value }));
  };

  const handleUpdateSubmit = (e) => {
    e.preventDefault();
    setUsers((prevUsers) =>
      prevUsers.map((user) => (user.id === editingUser.id ? { ...formData } : user))
    );
    setEditingUser(null); // Close the form after updating
    console.log(`User with ID ${formData.id} updated`);
  };

  return (
    <div className="search-results-container">
      <h2>Search Results</h2>
      <button onClick={handleBackClick}>← Back</button>
      {editingUser ? (
        <form onSubmit={handleUpdateSubmit}>
          <h3>Update User</h3>
          <label>
            First Name:
            <input type="text" name="firstName" value={formData.firstName} onChange={handleInputChange} required />
          </label>
          <label>
            Last Name:
            <input type="text" name="lastName" value={formData.lastName} onChange={handleInputChange} required />
          </label>
          <label>
            Middle Name:
            <input type="text" name="middleName" value={formData.middleName} onChange={handleInputChange} />
          </label>
          <label>
            Suffix:
            <input type="text" name="suffix" value={formData.suffix} onChange={handleInputChange} />
          </label>
          <label>
            Maiden Name:
            <input type="text" name="maidenName" value={formData.maidenName} onChange={handleInputChange} />
          </label>
          <label>
            Nickname:
            <input type="text" name="nickname" value={formData.nickname} onChange={handleInputChange} />
          </label>
          <label>
            Gender:
            <input type="text" name="gender" value={formData.gender} onChange={handleInputChange} />
          </label>
          <label>
            Tech Alum Chapter:
            <input type="text" name="techAlumChapter" value={formData.techAlumChapter} onChange={handleInputChange} />
          </label>
          <label>
            Class Year:
            <input type="number" name="classYear" value={formData.classYear} onChange={handleInputChange} required />
          </label>
          <label>
            Grad Year:
            <input type="number" name="gradYear" value={formData.gradYear} onChange={handleInputChange} required />
          </label>
          <label>
            Grad Semester:
            <input type="text" name="gradSemester" value={formData.gradSemester} onChange={handleInputChange} />
          </label>
          <button type="submit">Update User</button>
          <button type="button" onClick={() => setEditingUser(null)}>Cancel</button>
        </form>
      ) : users.length > 0 ? (
        <table className="search-results-table">
          <thead>
            <tr>
              <th>ID</th>
              <th>First Name</th>
              <th>Last Name</th>
              <th>Middle Name</th>
              <th>Suffix</th>
              <th>Maiden Name</th>
              <th>Nickname</th>
              <th>Gender</th>
              <th>Tech Alum Chapter</th>
              <th>Class Year</th>
              <th>Grad Year</th>
              <th>Grad Semester</th>
              <th>Degrees</th>
              <th>Addresses</th>
              <th>Contacts</th>
              <th>Involvements</th>
              <th>Actions</th> {/* Added Actions header */}
            </tr>
          </thead>
          <tbody>
            {users.map((user) => (
              <tr key={user.id}>
                <td>{user.id}</td>
                <td>{user.firstName}</td>
                <td>{user.lastName}</td>
                <td>{user.middleName}</td>
                <td>{user.suffix}</td>
                <td>{user.maidenName}</td>
                <td>{user.nickname}</td>
                <td>{user.gender}</td>
                <td>{user.techAlumChapter}</td>
                <td>{user.classYear}</td>
                <td>{user.gradYear}</td>
                <td>{user.gradSemester}</td>
                <td>
                  {user.degrees.map((degree, index) => (
                    <div key={index}>
                      {degree.degreeType} in {degree.subject} ({degree.degreeYear})
                    </div>
                  ))}
                </td>
                <td>
                  {user.addresses.map((address, index) => (
                    <div key={index}>
                      {address.address1}, {address.city}, {address.state} {address.zipCode}
                    </div>
                  ))}
                </td>
                <td>
                  {user.contacts.map((contact, index) => (
                    <div key={index}>
                      {contact.contactNumber} ({contact.contactType})
                    </div>
                  ))}
                </td>
                <td>
                  {user.involvements.join(", ")}
                </td>
                <td>
                  <div className="button-container">
                    <button 
                      className="update-button" 
                      onClick={() => handleUpdateUser(user.id)}
                    >
                      Update
                    </button>
                    <button 
                      className="delete-button" 
                      onClick={() => handleDeleteUser(user.id)}
                    >
                      Delete
                    </button>
                  </div>
                </td>
              </tr>
            ))}
          </tbody>
        </table>
      ) : (
        <p>No results found.</p>
      )}
    </div>
  );
};

export default SearchResults;
